// C++ Sample file to test ActiveX Server Object CornerDetector.CoCornerDetector

#include <stdio.h>
#include <conio.h>
//REQUIRED FOR COM EXTENDED FUNCTIONS
#define _WIN32_WINNT 0x0400
//LOAD TYPE INFORMATION DATA
#import "c:\code\c\matlabax\LogPolarMapper\LogPolarMapper.tlb" no_namespace named_guids
#import "c:\code\c\modules\Camera\Camera.tlb" no_namespace named_guids
#import "c:\code\c\modules\LogPolar\LogPolar.tlb" no_namespace named_guids

//THIS PROJECT REQUIRES MICROSOFT VISION SDK. 
//PLEASE MAKE SURE TO HAVE IT INSTALLED ON YOUR MACHINE
#include "viscore.h"

main()
{
	//INITIALIZE COM
	HRESULT hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
	if(FAILED(hr))
	{
		printf("Unable to Initialize COM\n");
		return -1;
	}


	long image_lines, image_columns;
	//CREATE ACTIVEX OBJECT
	ILogPolarMapper *ptr;
	hr = CoCreateInstance(	CLSID_CoLogPolarMapper, NULL, CLSCTX_INPROC_SERVER, 
							IID_ILogPolarMapper, (void**)&ptr);	
	if(FAILED(hr))
	{
		printf("Unable to Create Object\n");
		return -1;
	}

	ICamera *cam;
	hr = ptr->QueryInterface(IID_ICamera, (void**)&cam );
	if(FAILED(hr))
	{
		printf("Unavailable interface\n");
		return -1;
	}

	ILogPolar *log;
	hr = ptr->QueryInterface(IID_ILogPolar, (void**)&log );
	if(FAILED(hr))
	{
		printf("Unavailable interface\n");
		return -1;
	}

	CVisGrayByteImage img1;
	printf("Reading grayscale image ...\n");
	if (!img1.FReadFile("test1.bmp"))
	{
		printf("Could not open image\n");
		return -1;
	}

	//CONFIGURE OBJECT COMPATIBLE WITH IMAGE
	image_lines = img1.Height();
	image_columns = img1.Width();	
	hr = cam->put_image_frame_lines(image_lines);
	hr = cam->put_image_frame_columns(image_columns);
	hr = cam->put_sensor_lines(64);
	hr = cam->put_sensor_columns(32);
	hr = cam->put_color_channels(1);
	hr = log->put_rho_min(10);


	//ALLOCATE THE LOGPOLARMAPPER
	hr = ptr->create_logpolar_mapper();
	if(FAILED(hr))
	{
		printf("Unable to Allocate Internal Object Resources\n");
		return -1;
	}

	long fovea_pixels1;
	hr = log->get_fovea_pix(&fovea_pixels1);

	//ASK FOR THE DIRECT ACCESS INTERFACE
	_ILogPolarMapper *ptr_direct;
	hr = ptr->QueryInterface(IID__ILogPolarMapper, (void**)&ptr_direct);
	if(FAILED(hr))
	{
		printf("Unable to access direct interface\n");
		return -1;
	}

	//CREATE IMAGE FOR LOGPOLAR ARRAY 
	CVisByteImage logpolarimg1;
	logpolarimg1.Allocate(32, 64);

	//ALLOCATE MEMORY SPACE FOR FOVEA ARRAY
	byte * fovea1 = (byte*)malloc(fovea_pixels1);

	//LOGMAP
	hr = ptr_direct->logmap8U(img1.MemBlock().PbData(), logpolarimg1.MemBlock().PbData(), fovea1);
	if(FAILED(hr))
	{
		printf("Direct interface function call failed\n");
		return -1;
	}

	//SHOWING LOGPOLAR IMAGE
	printf("Displaying images\n");
	CVisImageWnd wlogpolar1(logpolarimg1);
	printf("Press any key to continue...\n");
	_getch();

	

	//CREATE IMAGE FOR REMAPPED ARRAY 
	CVisByteImage remappedimg1;
	remappedimg1.Allocate(img1.Rect());
	
	//INVMAP
	hr = ptr_direct->invmap8U(logpolarimg1.MemBlock().PbData(), fovea1, remappedimg1.MemBlock().PbData());
	if(FAILED(hr))
	{
		printf("Direct interface function call failed\n");
		return -1;
	}

	//SHOWING LOGPOLAR IMAGE
	printf("Displaying images\n");
	CVisImageWnd wremappedimg1(remappedimg1);
	printf("Press any key to continue...\n");
	_getch();


	//THE SAME FOR COLOR IMAGE

	CVisRGBAByteImage img2;
	printf("Reading image 2...\n");
	if (!img2.FReadFile("test2.bmp"))
	{
		printf("Could not open image 2\n");
		return -1;
	}

	//CONFIGURE OBJECT COMPATIBLE WITH IMAGE
	image_lines = img2.Height();
	image_columns = img2.Width();
	hr = cam->put_image_frame_lines(image_lines);
	hr = cam->put_image_frame_columns(image_columns);
	hr = cam->put_sensor_lines(64);
	hr = cam->put_sensor_columns(32);
	hr = cam->put_color_channels(4);
	hr = log->put_rho_min(10);


	//REALLOCATE THE LOGPOLARMAPPER
	hr = ptr->create_logpolar_mapper();
	if(FAILED(hr))
	{
		printf("Unable to Allocate Internal Object Resources\n");
		return -1;
	}

	long fovea_pixels2;
	hr = log->get_fovea_pix(&fovea_pixels2);


	//CREATE IMAGE FOR LOGPOLAR ARRAY 
	CVisRGBAByteImage logpolarimg2;
	logpolarimg2.Allocate(32, 64);

	//ALLOCATE MEMORY SPACE FOR FOVEA ARRAY
	byte * fovea2 = (byte*)malloc(fovea_pixels2*4);

	//LOGMAP
	hr = ptr_direct->logmap8U_AC4(img2.MemBlock().PbData(), logpolarimg2.MemBlock().PbData(), fovea2);
	if(FAILED(hr))
	{
		printf("Direct interface function call failed\n");
		return -1;
	}

	//SHOWING LOGPOLAR IMAGE
	printf("Displaying images\n");
	CVisImageWnd wlogpolar2(logpolarimg2);
	printf("Press any key to continue...\n");
	_getch();

	

	//CREATE IMAGE FOR REMAPPED ARRAY 
	CVisRGBAByteImage remappedimg2;
	remappedimg2.Allocate(img2.Rect());
	
	//INVMAP
	hr = ptr_direct->invmap8U_AC4(logpolarimg2.MemBlock().PbData(), fovea2, remappedimg2.MemBlock().PbData());
	if(FAILED(hr))
	{
		printf("Direct interface function call failed\n");
		return -1;
	}

	//SHOWING LOGPOLAR IMAGE
	printf("Displaying images\n");
	CVisImageWnd wremappedimg2(remappedimg2);
	printf("Press any key to continue...\n");
	_getch();



	//CLOSE EVERYTHING
	ptr->destroy_logpolar_mapper();
	ptr_direct->Release();
	cam->Release();
	log->Release();
	ptr->Release();
	free(fovea1);
	free(fovea2);



	CoUninitialize();
	
	printf("Done\n");
	return 0;
}